/**
 * Created on:		15 Sep 2010
 * @author:			Libor Kadlec
 *
 * Copyright (c) Changing Worlds Ltd. 2010.
*/
package com.amdocs.ycsb;

import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageConsumer;
import javax.jms.Queue;
import javax.jms.Session;

import org.hornetq.api.core.TransportConfiguration;
import org.hornetq.core.remoting.impl.netty.NettyConnectorFactory;
import org.hornetq.jms.client.HornetQConnectionFactory;
import org.hornetq.jms.client.HornetQQueue;

import com.yahoo.ycsb.DBException;

public class HornetQClient extends JMSClient {
    
    private static Connection connection;
    
    @Override
    public void init() throws DBException {
        synchronized (HornetQClient.class) {
            if (connection == null) {
                Properties p = getProperties();
                String host = p.getProperty("host");
                String port = p.getProperty("port");
                ConnectionFactory connectionFactory;
                if (host != null && port != null) {
                    Map<String, Object> params = new HashMap<String, Object>(2);
                    params.put("host", host);
                    params.put("port", port);
                    TransportConfiguration tc = new TransportConfiguration(NettyConnectorFactory.class.getName(), params);
                    connectionFactory = new HornetQConnectionFactory(tc);
                } else {
                    connectionFactory = new HornetQConnectionFactory("231.7.7.7", 9876);
                }
                try {
                    connection = connectionFactory.createConnection();
                    if (host == null && port == null) {
                        Thread.sleep(5000);  // Wait for servers to be discovered
                    }
                    connection.start();
                } catch (Exception e) {
                    throw new DBException(e);
                }
            }
        }
        super.init();
    }
    
    @Override
    protected Connection getConnection() {
        return connection;
    }

    @Override
    protected Queue createQueue(String name) {
        return new HornetQQueue(name);
    }
    
    public static void closeConnection() {
        try {
            if (connection != null) {
                connection.close();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    
    public static int drainQueue(String host, String queueName) {
        Connection con = null;
        try {
            Map<String, Object> params = new HashMap<String, Object>(2);
            params.put("host", host);
            params.put("port", "5445");
            TransportConfiguration tc = new TransportConfiguration(NettyConnectorFactory.class.getName(), params);
            ConnectionFactory cf = new HornetQConnectionFactory(tc);
            con = cf.createConnection();
            con.start();
            Session drainSession = con.createSession(false, Session.AUTO_ACKNOWLEDGE);
            MessageConsumer consumer = drainSession.createConsumer(new HornetQQueue(queueName));
            int count = 0;
            Message message = null;
            do {
                message = consumer.receive(3000);
                if (message != null) {
                    message.acknowledge();
                    count++;
                }
            }
            while (message != null);
            return count;
        } catch (JMSException e) {
            // It can be thrown when server is not running. Basically we don't really care if something happens here.
        } finally {
            if (con != null) {
                try {
                    con.close();
                } catch (JMSException e) {
                }
            }
        }
        return 0;
    }

}
